% [x,ret] = sdprealize(K,P,epsilon,solver)
%
%   solve SDP with solver 'solver' to find a realization of the partial 
%   EDM P in R^K epsilon is used to go from Gram matrix to realization.
%   SDP model by L.Liberti (from Y. Ye) coded by A.A. Ahmadi & L. Liberti
%   x = the realization
%   ret: a return data structure with the following fields:
%     realization = x
%     error = the error between P and eucldist(x) over P ~= -1
%     rank = rank of the Gram matrix corresponding to completion of P
%     eigenvalues = eigenvalues of the Gram matrix
%     diagnostic = diagnostic messages from SDP solver  

function [x,ret] = sdprealize(K,P,epsilon,solver)
  if (nargin < 3)
    epsilon = 0.0001;
  end
  if (nargin < 4)
    %solver = 'mosek';
    %solver = 'sedumi';
    solver = 'sdpt3';
  end
  opts = sdpsettings( 'solver', solver);
  %tic;
  [n,n] = size(P);

  Y = sdpvar(n,n);
  
  %% schur complement (useless when X do not appear in problem)
  %X = sdpvar(K,n);
  %M = [ eye(K) X; X' Y ];
  %F = [ M >= 0 ];
  
  %% no schur complement
  F = [Y >= 0];
  obj = 0;
  
  %% construct the constraint matrix and objective function
  for i = 1:n-1
    for j = i+1:n
      if (P(i,j) ~= -1)
        F = F + [Y(i,i) + Y(j,j) - 2*Y(i,j) == P(i,j)^2];
        %F = F + [Y(i,i) + Y(j,j) - 2*Y(i,j) >= P(i,j)^2];
        obj = obj + (Y(i,i) + Y(j,j) - 2*Y(i,j)); %% for ineq
      end
    end
  end
  obj = obj + 0.001*trace(Y);
  %obj = trace(Y);
  
  %% solve and collect solution
  sdpout = solvesdp(F,obj,opts);
  G = double(Y);
  [V,lambda] = eigs(G,K,'LM');
  x = real(eps2zero(lambda.^(1/2) * V', epsilon));
  derr = pedmerror(P, eucldist(x));
  grk = rank(G,epsilon);
  mx = mde(P,x);
  lx = lde(P,x);
  ov = double(obj);
  %cpu = toc;
  cpu = -1;
  fprintf('sdp: cpu=%.2f mde=%.2f lde=%.2f rk=%d obj=%.4f\n',cpu,mx,lx,grk,ov);  
  ret = struct('realization', x, 'error', derr, 'rank', grk, ...
      'eigenvalues', eig(G),'diagnostic', sdpout, 'sol', G, 'obj', ov, ...
      'mde', mx, 'lde', lx);
end

